"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * computes frequencies of the values in the data elements
 * If enabled, keeps the ebject with top 10 frequencies always available
 *
 * it also computes distinct counts (unique/duplicate/distinct/nonUnique)
 */

var FrequencyMetrics = function () {
    /**
     * initialize frequency object
     * @param options
     * elements: array of the same size as data object
     * computeTmp: keep top n frequencies ready and accessible
     * tmpElements: max no of elements in tmp array
     */

    function FrequencyMetrics() {
        var options = arguments.length <= 0 || arguments[0] === undefined ? { elements: [], computeTmp: true, tmpElements: 10 } : arguments[0];

        _classCallCheck(this, FrequencyMetrics);

        var elems = options.elements || [];
        //frequencyObject that stores frequencies (index is the index of the element in elements)
        this.freqObject = this._createFreqObject(elems);
        //each field has its own frequency map and tmp freqyency map. Initialized with min value.
        this.fields = elems.map(function (element) {
            return { freqMap: new Map(), freqTmp: new Map(), tmpMinVal: { key: null, value: 0 } };
        });
        this.maxTmp = options.tmpElements || 10;
        this.distinctCount = elems.map(function (elem) {
            return { nonUnique: 0, unique: 0, duplicate: 0, distinct: 0 };
        });
        if (typeof options.computeTmp !== "undefined") {
            this.computeTmp = options.computeTmp;
        } else {
            this.computeTmp = true;
        }
    }

    _createClass(FrequencyMetrics, [{
        key: "_createFreqObject",
        value: function _createFreqObject(elements) {
            return elements.map(function (elem) {
                return { freq: [] };
            });
        }

        /**
         * add new "record" or attribute set to check
         * @param elements array of data elements
         */

    }, {
        key: "updateFrequencies",
        value: function updateFrequencies(elements) {
            if (elements.length == this.freqObject.length) {
                for (var i = 0; i < this.freqObject.length; i++) {
                    this._updateFreq(elements[i], i);
                }
            } else {
                console.error("FrequencyMetrics: checked object does not have the same length as initialized Metrics, ignoring");
            }
        }

        /**
         * update single value for single field .. compute frequency
         * @param field
         * @param index
         * @private
         */

    }, {
        key: "_updateFreq",
        value: function _updateFreq(field, index) {
            var map = this.fields[index].freqMap;
            var tmp = this.fields[index].freqTmp;
            var tmpMinVal = this.fields[index].tmpMinVal;
            var val = map.get(field.value);
            //check if the value is already in the list, if yes, update count, if not, add
            if (typeof val !== 'undefined') {
                map.set(field.value, val + 1);
                val = val + 1;
            } else {
                map.set(field.value, 1);
                val = 1;
            }
            //the tmp list (top n values)
            if (this.computeTmp) {
                var tmpVl = tmp.get(field.value);
                //check if the value exists in the list
                if (typeof tmpVl !== 'undefined') {
                    //add one to the value from the list
                    tmp.set(field.value, val);
                    //if the value was minimum in the list, check if it is more than any other value after update
                    if (field.value == tmpMinVal.key) {
                        var min = tmpMinVal.value;
                        var minKey = field.value;
                        //check if any value from the list has lower frequency
                        var _iteratorNormalCompletion = true;
                        var _didIteratorError = false;
                        var _iteratorError = undefined;

                        try {
                            for (var _iterator = tmp.keys()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                                var key = _step.value;

                                if (tmp.get(key) < min) {
                                    min = tmp.get(key);
                                    minKey = key;
                                }
                            }
                            //if lower frequency found, set is as minimum for this field
                        } catch (err) {
                            _didIteratorError = true;
                            _iteratorError = err;
                        } finally {
                            try {
                                if (!_iteratorNormalCompletion && _iterator.return) {
                                    _iterator.return();
                                }
                            } finally {
                                if (_didIteratorError) {
                                    throw _iteratorError;
                                }
                            }
                        }

                        tmpMinVal = { key: minKey, value: min };
                    }
                } else {
                    //value was not found in the list
                    //add new value to the list if size of tempList is under max acceptable
                    //set this value as minimum automatically
                    if (tmp.size < this.maxTmp) {
                        tmp.set(field.value, val);
                        if (tmp.size == 1) {
                            tmpMinVal = { key: field.value, value: val };
                        }
                        //if value is not in the list and list is "full"
                    } else {
                            //check if frequency is higher than the minimum in the list. If yes, delete minimum and add the new value as the minimum
                            if (val >= tmpMinVal.value) {
                                if (tmpMinVal.key != null) {
                                    tmp.delete(tmpMinVal.key);
                                }
                                tmp.set(field.value, val);
                                tmpMinVal = { key: field.value, value: val };
                            }
                        }
                }
            }
            //check unique/distinct/nonunique counts
            if (val == 1) {
                //this.distinctCount = {nonUnique: 0, unique: 0,duplicate: 0, distinct: 0};
                this.distinctCount[index].distinct++;
                this.distinctCount[index].unique++;
            } else {
                if (val == 2) {
                    this.distinctCount[index].nonUnique++;
                    this.distinctCount[index].unique--;
                }
                this.distinctCount[index].duplicate++;
            }
            //update metrics of the object
            this.fields[index].freqMap = map;
            this.fields[index].freqTmp = tmp;
            this.fields[index].tmpMinVal = tmpMinVal;
        }
    }, {
        key: "getFreqTopObject",
        value: function getFreqTopObject() {
            return this.fields.map(function (field) {
                return {
                    freq: Array.from(field.freqTmp).map(function (elem) {
                        return { label: elem[0], value: elem[1] };
                    })
                };
            });
        }
    }, {
        key: "getFreqObject",
        value: function getFreqObject() {
            return this.fields.map(function (field) {
                return {
                    freq: Array.from(field.freqMap).map(function (elem) {
                        return { label: elem[0], value: elem[1] };
                    })
                };
            });
        }
    }, {
        key: "getDistinctOpbject",
        value: function getDistinctOpbject() {
            return this.distinctCount;
        }
    }]);

    return FrequencyMetrics;
}();

exports.default = FrequencyMetrics;