"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.MaskMetrics = exports.FrequencyMetrics = exports.NullMetrics = exports.DataStatistics = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _dataStatisticsNullMetrics = require("./data-statistics-nullMetrics");

var _dataStatisticsNullMetrics2 = _interopRequireDefault(_dataStatisticsNullMetrics);

var _dataStatisticsFrequencyMetrics = require("./data-statistics-frequencyMetrics");

var _dataStatisticsFrequencyMetrics2 = _interopRequireDefault(_dataStatisticsFrequencyMetrics);

var _dataStatisticsMasks = require("./data-statistics-masks");

var _dataStatisticsMasks2 = _interopRequireDefault(_dataStatisticsMasks);

require("babel-polyfill");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DataStatistics = exports.DataStatistics = function () {
    function DataStatistics() {
        var options = arguments.length <= 0 || arguments[0] === undefined ? { elements: [], nullMetrics: { considerEmpty: true }, frequencyMetrics: { computeTmp: true, tmpElements: 10 }, maskMetrics: { computeTmp: true, tmpElements: 10 } } : arguments[0];

        _classCallCheck(this, DataStatistics);

        var elems = options.elements || [];
        this.nullMetrics = new _dataStatisticsNullMetrics2.default(Object.assign(options.nullMetrics || {}, { elements: elems }));
        this.frequencyMetrics = new _dataStatisticsFrequencyMetrics2.default(Object.assign(options.frequencyMetrics || {}, { elements: elems }));
        this.maskMetrics = new _dataStatisticsMasks2.default(Object.assign(options.maskMetrics || {}, { elements: elems }));
        this.count = 0;
    }

    /**
     * add new data row to the statistics. Call this for every data row
     * @param dataObject = [{vlaue: "someting"},{value: "...
     */


    _createClass(DataStatistics, [{
        key: "updateStatistics",
        value: function updateStatistics(dataObject) {
            this.count++;
            this.nullMetrics.checkNulls(dataObject);
            this.frequencyMetrics.updateFrequencies(dataObject);
            this.maskMetrics.updateStatistics(dataObject);
        }

        /**
         * Gets an array of frequencies (top n) for the speciffied field (by index)
         * @param index
         * @returns []
         */

    }, {
        key: "getTopFrequencyObjectForField",
        value: function getTopFrequencyObjectForField() {
            var index = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

            return this.frequencyMetrics.getFreqTopObject()[index];
        }

        /**
         * Returns array of fields with their top n frequencies
         * @returns {*}
         */

    }, {
        key: "getTopFrequencyObject",
        value: function getTopFrequencyObject() {
            return this.frequencyMetrics.getFreqTopObject();
        }

        /**
         * Gets an array of frequencies of mask analysis (top n) for the speciffied field (by index)
         * @param index
         * @returns []
         */

    }, {
        key: "getTopMaskFrequencyObjectForField",
        value: function getTopMaskFrequencyObjectForField() {
            var index = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

            return this.maskMetrics.getFreqTopObject()[index];
        }

        /**
         * Returns array of fields with their top n frequencies of their masks
         * @returns {*}
         */

    }, {
        key: "getTopMaskFrequencyObject",
        value: function getTopMaskFrequencyObject() {
            return this.maskMetrics.getFreqTopObject();
        }
        /**
         * Returns an ratio of nulls to not nulls for speciffic field
         * @param index
         * @returns {*}
         */

    }, {
        key: "getNullObjectForField",
        value: function getNullObjectForField() {
            var index = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

            return this.nullMetrics.nullObject[index];
        }

        /**
         * Returns an array with null objects for all the fields
         * @returns {*}
         */

    }, {
        key: "getNullObject",
        value: function getNullObject() {
            return this.nullMetrics.nullObject;
        }

        /**
         * Returns full object with all frequencies for the speciffic field
         * @param index
         * @returns {*}
         */

    }, {
        key: "getFrequencyObjectForField",
        value: function getFrequencyObjectForField() {
            var index = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

            return this.frequencyMetrics.getFreqObject()[index];
        }

        /**
         * Returns array of all fields with their frequency objects
         * @returns {*}
         */

    }, {
        key: "getFrequencyObject",
        value: function getFrequencyObject() {
            return this.frequencyMetrics.getFreqObject();
        }
        /**
         * Returns full object with all mask analysis frequencies for the speciffic field
         * @param index
         * @returns {*}
         */

    }, {
        key: "getMaskFrequencyObjectForField",
        value: function getMaskFrequencyObjectForField() {
            var index = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

            return this.maskMetrics.getFreqObject()[index];
        }

        /**
         * Returns full object with all mask frequencies for all the fields
         * @returns {*}
         */

    }, {
        key: "getMaskFrequencyObject",
        value: function getMaskFrequencyObject() {
            return this.maskMetrics.getFreqObject();
        }
        /**
         * Returns the object containing distinct/unique/diplicate count for speciffic field
         * @param index
         * @returns {*}
         */

    }, {
        key: "getDistinctCountsForField",
        value: function getDistinctCountsForField() {
            var index = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

            return this.frequencyMetrics.getDistinctOpbject()[index];
        }

        /**
         * Returns oth object containing distinct/unique/duplicate count for all fields
         * @returns {*}
         */

    }, {
        key: "getDistinctCounts",
        value: function getDistinctCounts() {
            return this.frequencyMetrics.getDistinctOpbject();
        }
        /**
         * Returns the object containing distinct/unique/duplicate count of masks for speciffic field
         * @param index
         * @returns {*}
         */

    }, {
        key: "getMasksDistinctCountsForField",
        value: function getMasksDistinctCountsForField() {
            var index = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

            return this.maskMetrics.getDistinctOpbject()[index];
        }

        /**
         * Returns the object containing distinct/unique/duplicate count of masks for all fields
         * @returns {*}
         */

    }, {
        key: "getMasksDistinctCounts",
        value: function getMasksDistinctCounts() {
            return this.maskMetrics.getDistinctOpbject();
        }
    }]);

    return DataStatistics;
}();

var NullMetrics = exports.NullMetrics = function (_NullM) {
    _inherits(NullMetrics, _NullM);

    function NullMetrics() {
        _classCallCheck(this, NullMetrics);

        return _possibleConstructorReturn(this, Object.getPrototypeOf(NullMetrics).apply(this, arguments));
    }

    return NullMetrics;
}(_dataStatisticsNullMetrics2.default);

;

var FrequencyMetrics = exports.FrequencyMetrics = function (_FrequencyM) {
    _inherits(FrequencyMetrics, _FrequencyM);

    function FrequencyMetrics() {
        _classCallCheck(this, FrequencyMetrics);

        return _possibleConstructorReturn(this, Object.getPrototypeOf(FrequencyMetrics).apply(this, arguments));
    }

    return FrequencyMetrics;
}(_dataStatisticsFrequencyMetrics2.default);

;

var MaskMetrics = exports.MaskMetrics = function (_MaskM) {
    _inherits(MaskMetrics, _MaskM);

    function MaskMetrics() {
        _classCallCheck(this, MaskMetrics);

        return _possibleConstructorReturn(this, Object.getPrototypeOf(MaskMetrics).apply(this, arguments));
    }

    return MaskMetrics;
}(_dataStatisticsMasks2.default);

;